@@ -8,7 +8,7 @@
 # This code is distributed under the GPLv3 license, a copy of
 # which is included in the root directory of this package.
 #
-import numpy
+import numpy as np
 import maptools
 import mrcfile
 import os.path
@@ -26,7 +26,7 @@ random.seed(0)
 
 def reconstruct(image_filename, rec_filename, microscope, simulation, device="gpu"):
     """
-    Reconstruct the volume using 3D CTF correction
+    Reconstruct the volume and use 3D CTF correction beforehand if the input image is uncorrected
 
     """
 
@@ -72,6 +72,57 @@ def reconstruct(image_filename, rec_filename, microscope, simulation, device="gp
     )
 
 
+def correct(
+    image_filename,
+    corrected_filename,
+    microscope,
+    simulation,
+    num_defocus=None,
+    device="gpu",
+):
+    """
+    Correct the images using 3D CTF correction
+
+    """
+
+    # Ensure mrc file
+    assert os.path.splitext(image_filename)[1] == ".mrc"
+
+    # Get the parameters for the CTF correction
+    nx = microscope.detector.nx
+    pixel_size = microscope.detector.pixel_size
+    energy = microscope.beam.energy
+    defocus = -microscope.lens.c_10
+    if num_defocus is None:
+        num_defocus = int((nx * pixel_size) / 100)
+
+    # Set the spherical aberration
+    if simulation["inelastic_model"] == "cc_corrected":
+        print("Setting spherical aberration to zero")
+        spherical_aberration = 0
+    else:
+        spherical_aberration = microscope.lens.c_30
+
+    astigmatism = microscope.lens.c_12
+    astigmatism_angle = microscope.lens.phi_12
+    phase_shift = 0
+
+    # Do the reconstruction
+    guanaco.correct_file(
+        input_filename=image_filename,
+        output_filename=corrected_filename,
+        centre=None,
+        energy=energy,
+        defocus=defocus,
+        num_defocus=num_defocus,
+        spherical_aberration=spherical_aberration,
+        astigmatism=astigmatism,
+        astigmatism_angle=astigmatism_angle,
+        phase_shift=phase_shift,
+        device=device,
+    )
+
+
 def average_particles(
     scan,
     sample_filename,
@@ -79,6 +130,7 @@ def average_particles(
     half_1_filename,
     half_2_filename,
     particle_size=0,
+    num_particles=0,
 ):
     """
     Average particles to compute averaged reconstruction
@@ -88,13 +140,13 @@ def average_particles(
     def rotate_array(data, rotation, offset):
 
         # Create the pixel indices
-        az = numpy.arange(data.shape[0])
-        ay = numpy.arange(data.shape[1])
-        ax = numpy.arange(data.shape[2])
-        x, y, z = numpy.meshgrid(az, ay, ax, indexing="ij")
+        az = np.arange(data.shape[0])
+        ay = np.arange(data.shape[1])
+        ax = np.arange(data.shape[2])
+        x, y, z = np.meshgrid(az, ay, ax, indexing="ij")
 
         # Create a stack of coordinates
-        xyz = numpy.vstack(
+        xyz = np.vstack(
             [
                 x.reshape(-1) - offset[0],
                 y.reshape(-1) - offset[1],
@@ -126,21 +178,21 @@ def average_particles(
     sample = parakeet.sample.load(sample_filename)
 
     # Get the sample centre
-    centre = numpy.array(sample.centre)
+    centre = np.array(sample.centre)
 
     # Read the reconstruction file
     tomo_file = mrcfile.mmap(rec_filename)
     tomogram = tomo_file.data
 
     # Get the size of the volume
-    voxel_size = numpy.array(
+    voxel_size = np.array(
         (
             tomo_file.voxel_size["x"],
             tomo_file.voxel_size["y"],
             tomo_file.voxel_size["z"],
         )
     )
-    size = numpy.array(tomogram.shape)[[2, 0, 1]] * voxel_size
+    size = np.array(tomogram.shape)[[2, 0, 1]] * voxel_size
 
     # Loop through the
     assert sample.number_of_molecules == 1
@@ -167,14 +219,15 @@ def average_particles(
             half_length = particle_size // 2
         length = 2 * half_length
         assert len(positions) == len(orientations)
-        num_particles = len(positions)
+        if num_particles <= 0:
+            num_particles = len(positions)
         print(
             "Averaging %d %s particles with box size %d" % (num_particles, name, length)
         )
 
         # Create the average array
-        half_1 = numpy.zeros(shape=(length, length, length), dtype="float32")
-        half_2 = numpy.zeros(shape=(length, length, length), dtype="float32")
+        half_1 = np.zeros(shape=(length, length, length), dtype="float32")
+        half_2 = np.zeros(shape=(length, length, length), dtype="float32")
         num_1 = 0
         num_2 = 0
 
@@ -187,7 +240,7 @@ def average_particles(
         for i, (position, orientation) in enumerate(zip(positions, orientations)):
 
             # Compute p within the volume
-            # start_position = numpy.array([0, scan["start_pos"], 0])
+            # start_position = np.array([0, scan["start_pos"], 0])
             p = position - (centre - size / 2.0)  # - start_position
             p[2] = size[2] - p[2]
             print(
@@ -200,9 +253,9 @@ def average_particles(
             )
 
             # Set the region to extract
-            x0 = numpy.floor(p).astype("int32") - half_length
-            x1 = numpy.floor(p).astype("int32") + half_length
-            offset = p - numpy.floor(p).astype("int32")
+            x0 = np.floor(p).astype("int32") - half_length
+            x1 = np.floor(p).astype("int32") + half_length
+            offset = p - np.floor(p).astype("int32")
 
             # Get the sub tomogram
             print("Getting sub tomogram")
@@ -213,8 +266,8 @@ def average_particles(
                 data = sub_tomo
 
                 # Reorder input vectors
-                offset = numpy.array(data.shape)[::-1] / 2 + offset[[1, 2, 0]]
-                rotation = -numpy.array(orientation)[[1, 2, 0]]
+                offset = np.array(data.shape)[::-1] / 2 + offset[[1, 2, 0]]
+                rotation = -np.array(orientation)[[1, 2, 0]]
                 rotation[1] = -rotation[1]
 
                 # Rotate the data
@@ -229,6 +282,10 @@ def average_particles(
                     half_2 += data
                     num_2 += 1
 
+            # Break if we have enough particles
+            if num_1 + num_2 >= num_particles:
+                break
+
         # Average the sub tomograms
         print("Averaging half 1 with %d particles" % num_1)
         print("Averaging half 2 with %d particles" % num_2)
@@ -263,13 +320,13 @@ def average_all_particles(
     def rotate_array(data, rotation, offset):
 
         # Create the pixel indices
-        az = numpy.arange(data.shape[0])
-        ay = numpy.arange(data.shape[1])
-        ax = numpy.arange(data.shape[2])
-        x, y, z = numpy.meshgrid(az, ay, ax, indexing="ij")
+        az = np.arange(data.shape[0])
+        ay = np.arange(data.shape[1])
+        ax = np.arange(data.shape[2])
+        x, y, z = np.meshgrid(az, ay, ax, indexing="ij")
 
         # Create a stack of coordinates
-        xyz = numpy.vstack(
+        xyz = np.vstack(
             [
                 x.reshape(-1) - offset[0],
                 y.reshape(-1) - offset[1],
@@ -301,21 +358,21 @@ def average_all_particles(
     sample = parakeet.sample.load(sample_filename)
 
     # Get the sample centre
-    centre = numpy.array(sample.centre)
+    centre = np.array(sample.centre)
 
     # Read the reconstruction file
     tomo_file = mrcfile.mmap(rec_filename)
     tomogram = tomo_file.data
 
     # Get the size of the volume
-    voxel_size = numpy.array(
+    voxel_size = np.array(
         (
             tomo_file.voxel_size["x"],
             tomo_file.voxel_size["y"],
             tomo_file.voxel_size["z"],
         )
     )
-    size = numpy.array(tomogram.shape)[[2, 0, 1]] * voxel_size
+    size = np.array(tomogram.shape)[[2, 0, 1]] * voxel_size
 
     # Loop through the
     assert sample.number_of_molecules == 1
@@ -348,7 +405,7 @@ def average_all_particles(
         )
 
         # Create the average array
-        average = numpy.zeros(shape=(length, length, length), dtype="float32")
+        average = np.zeros(shape=(length, length, length), dtype="float32")
         num = 0
 
         # Sort the positions and orientations by y
@@ -360,7 +417,7 @@ def average_all_particles(
         for i, (position, orientation) in enumerate(zip(positions, orientations)):
 
             # Compute p within the volume
-            start_position = numpy.array([0, scan["start_pos"], 0])
+            start_position = np.array([0, scan["start_pos"], 0])
             p = position - (centre - size / 2.0)  # - start_position
             p[2] = size[2] - p[2]
             print(
@@ -373,9 +430,9 @@ def average_all_particles(
             )
 
             # Set the region to extract
-            x0 = numpy.floor(p).astype("int32") - half_length
-            x1 = numpy.floor(p).astype("int32") + half_length
-            offset = p - numpy.floor(p).astype("int32")
+            x0 = np.floor(p).astype("int32") - half_length
+            x1 = np.floor(p).astype("int32") + half_length
+            offset = p - np.floor(p).astype("int32")
 
             # Get the sub tomogram
             print("Getting sub tomogram")
@@ -386,8 +443,8 @@ def average_all_particles(
                 data = sub_tomo
 
                 # Reorder input vectors
-                offset = numpy.array(data.shape)[::-1] / 2 + offset[[1, 2, 0]]
-                rotation = -numpy.array(orientation)[[1, 2, 0]]
+                offset = np.array(data.shape)[::-1] / 2 + offset[[1, 2, 0]]
+                rotation = -np.array(orientation)[[1, 2, 0]]
                 rotation[1] = -rotation[1]
 
                 # Rotate the data
@@ -425,13 +482,13 @@ def extract_particles(
     def rotate_array(data, rotation, offset):
 
         # Create the pixel indices
-        az = numpy.arange(data.shape[0])
-        ay = numpy.arange(data.shape[1])
-        ax = numpy.arange(data.shape[2])
-        x, y, z = numpy.meshgrid(az, ay, ax, indexing="ij")
+        az = np.arange(data.shape[0])
+        ay = np.arange(data.shape[1])
+        ax = np.arange(data.shape[2])
+        x, y, z = np.meshgrid(az, ay, ax, indexing="ij")
 
         # Create a stack of coordinates
-        xyz = numpy.vstack(
+        xyz = np.vstack(
             [
                 x.reshape(-1) - offset[0],
                 y.reshape(-1) - offset[1],
@@ -463,21 +520,21 @@ def extract_particles(
     sample = parakeet.sample.load(sample_filename)
 
     # Get the sample centre
-    centre = numpy.array(sample.centre)
+    centre = np.array(sample.centre)
 
     # Read the reconstruction file
     tomo_file = mrcfile.mmap(rec_filename)
     tomogram = tomo_file.data
 
     # Get the size of the volume
-    voxel_size = numpy.array(
+    voxel_size = np.array(
         (
             tomo_file.voxel_size["x"],
             tomo_file.voxel_size["y"],
             tomo_file.voxel_size["z"],
         )
     )
-    size = numpy.array(tomogram.shape)[[2, 0, 1]] * voxel_size
+    size = np.array(tomogram.shape)[[2, 0, 1]] * voxel_size
 
     # Loop through the
     assert sample.number_of_molecules == 1
@@ -511,7 +568,7 @@ def extract_particles(
 
         # Create the average array
         extract_map = []
-        particle_instance = numpy.zeros(shape=(length, length, length), dtype="float32")
+        particle_instance = np.zeros(shape=(length, length, length), dtype="float32")
         num = 0
 
         # Sort the positions and orientations by y
@@ -523,7 +580,7 @@ def extract_particles(
         for i, (position, orientation) in enumerate(zip(positions, orientations)):
 
             # Compute p within the volume
-            start_position = numpy.array([0, scan["start_pos"], 0])
+            start_position = np.array([0, scan["start_pos"], 0])
             p = position - (centre - size / 2.0)  # - start_position
             p[2] = size[2] - p[2]
             print(
@@ -536,9 +593,9 @@ def extract_particles(
             )
 
             # Set the region to extract
-            x0 = numpy.floor(p).astype("int32") - half_length
-            x1 = numpy.floor(p).astype("int32") + half_length
-            offset = p - numpy.floor(p).astype("int32")
+            x0 = np.floor(p).astype("int32") - half_length
+            x1 = np.floor(p).astype("int32") + half_length
+            offset = p - np.floor(p).astype("int32")
 
             # Get the sub tomogram
             print("Getting sub tomogram")
@@ -549,8 +606,8 @@ def extract_particles(
                 data = sub_tomo
 
                 # Reorder input vectors
-                offset = numpy.array(data.shape)[::-1] / 2 + offset[[1, 2, 0]]
-                rotation = -numpy.array(orientation)[[1, 2, 0]]
+                offset = np.array(data.shape)[::-1] / 2 + offset[[1, 2, 0]]
+                rotation = -np.array(orientation)[[1, 2, 0]]
                 rotation[1] = -rotation[1]
 
                 # Rotate the data
@@ -564,7 +621,7 @@ def extract_particles(
 
         # Average the sub tomograms
         print("Extracting %d particles" % num)
-        extract_map = numpy.array(extract_map)
+        extract_map = np.array(extract_map)
 
         # from matplotlib import pylab
         # pylab.imshow(average[half_length, :, :])
