@@ -69,7 +69,7 @@ def rebin(data, shape):
     x1 = x0 + shape[1]
     y1 = y0 + shape[0]
     y, x = numpy.mgrid[0 : data.shape[0], 0 : data.shape[1]]
-    r = (y - yc) ** 2 / yh ** 2 + (x - xc) ** 2 / xh ** 2
+    r = (y - yc) ** 2 / yh**2 + (x - xc) ** 2 / xh**2
     mask = r < 1.0
     f = f * mask
     f = f[y0:y1, x0:x1]
@@ -89,6 +89,8 @@ def filter_image(data, pixel_size, resolution, shape):
         shape (str): The filter shape
 
     """
+    if pixel_size == 0:
+        pixel_size = 1
     f = numpy.fft.fft2(data)
     f = numpy.fft.fftshift(f)
     yc, xc = data.shape[0] // 2, data.shape[1] // 2
@@ -102,7 +104,8 @@ def filter_image(data, pixel_size, resolution, shape):
     if shape == "square":
         g = r < 1.0 / resolution
     elif shape == "guassian":
-        g = numpy.exp(-0.5 * r ** 2 * resolution ** 2)
+        g = numpy.exp(-0.5 * r**2 * resolution**2)
+    f = f * g
     f = numpy.fft.ifftshift(f)
     d = numpy.fft.ifft2(f)
     return d.real
@@ -209,6 +212,14 @@ def export(argv=None):
         dest="vmax",
         help="The maximum pixel value when exporting to an image",
     )
+    parser.add_argument(
+        "--sort",
+        type=str,
+        default=None,
+        dest="sort",
+        choices=["angle"],
+        help="Sort the images",
+    )
 
     # Parse the arguments
     args = parser.parse_args(argv)
@@ -257,6 +268,11 @@ def export(argv=None):
         else:
             random.shuffle(indices)
 
+    # Sort the images
+    if args.sort is not None:
+        if args.sort == "angle":
+            indices = sorted(indices, key=lambda i: reader.angle[indices[i]])
+
     # Get the region of interest
     if args.roi is not None:
         x0, y0, x1, y1 = tuple(map(int, args.roi.split(",")))
@@ -335,6 +351,8 @@ def export(argv=None):
         angle = reader.angle[i]
         position = reader.position[i]
         pixel_size = reader.pixel_size
+        drift = reader.drift[i]
+        defocus = reader.defocus[i]
 
         # Rotate if necessary
         if args.rot90:
@@ -368,6 +386,10 @@ def export(argv=None):
         writer.data[j, :, :] = image
         writer.angle[j] = angle
         writer.position[j] = position
+        if drift is not None:
+            writer.drift[j] = drift
+        if defocus is not None:
+            writer.defocus[j] = defocus
 
     # Update the writer
     writer.update()
