@@ -101,6 +101,94 @@ def reconstruct(args=None):
     logger.info("Time taken: %.2f seconds" % (time.time() - start_time))
 
 
+def correct(args=None):
+    """
+    Correct the images using 3D CTF correction
+
+    """
+
+    # Get the start time
+    start_time = time.time()
+
+    # Create the argument parser
+    parser = argparse.ArgumentParser(description="3D CTF correction of the images")
+
+    # Add some command line arguments
+    parser.add_argument(
+        "-c",
+        "--config",
+        type=str,
+        default=None,
+        dest="config",
+        help="The yaml file to configure the simulation",
+    )
+    parser.add_argument(
+        "-d",
+        "--device",
+        choices=["cpu", "gpu"],
+        default=None,
+        dest="device",
+        help="Choose the device to use",
+    )
+    parser.add_argument(
+        "-i",
+        "--image",
+        type=str,
+        default="image.mrc",
+        dest="image",
+        help="The filename for the image",
+    )
+    parser.add_argument(
+        "-cr",
+        "--corrected",
+        type=str,
+        default="corrected_image.mrc",
+        dest="corrected",
+        help="The filename for the corrected image",
+    )
+    parser.add_argument(
+        "-ndf",
+        "--num-defocus",
+        type=int,
+        default=None,
+        dest="num_defocus",
+        help="Number of defoci that correspond to different depths through for which the sample will be 3D CTF corrected",
+    )
+
+    # Parse the arguments
+    args = parser.parse_args(args=args)
+
+    # Configure some basic logging
+    parakeet.command_line.configure_logging()
+
+    # Set the command line args in a dict
+    command_line = {}
+    if args.device is not None:
+        command_line["device"] = args.device
+
+    # Load the full configuration
+    config = parakeet.config.load(args.config, command_line)
+
+    # Print some options
+    parakeet.config.show(config)
+
+    # Create the microscope
+    microscope = parakeet.microscope.new(**config["microscope"])
+
+    # Do the reconstruction
+    parakeet.analyse.correct(
+        args.image,
+        args.corrected,
+        microscope=microscope,
+        simulation=config["simulation"],
+        num_defocus=args.num_defocus,
+        device=config["device"],
+    )
+
+    # Write some timing stats
+    logger.info("Time taken: %.2f seconds" % (time.time() - start_time))
+
+
 def average_particles(args=None):
     """
     Perform sub tomogram averaging
@@ -162,6 +250,14 @@ def average_particles(args=None):
         dest="particle_size",
         help="The size of for the particles extracted",
     )
+    parser.add_argument(
+        "-n",
+        "--num_particles",
+        type=int,
+        default=0,
+        dest="num_particles",
+        help="The nunber of particles to use",
+    )
 
     # Parse the arguments
     args = parser.parse_args(args=args)
@@ -186,6 +282,7 @@ def average_particles(args=None):
         args.half1,
         args.half2,
         args.particle_size,
+        args.num_particles,
     )
 
     # Write some timing stats
