@@ -9,7 +9,7 @@
 # which is included in the root directory of this package.
 #
 import h5py
-import numpy
+import numpy as np
 import mrcfile
 import os
 import PIL.Image
@@ -67,12 +67,12 @@ class Writer(object):
         return self._position
 
     @property
-    def shift(self):
+    def drift(self):
         """
-        The shift property
+        The drift property
 
         """
-        return self._shift
+        return self._drift
 
     @property
     def pixel_size(self):
@@ -82,6 +82,14 @@ class Writer(object):
         """
         return self._pixel_size
 
+    @property
+    def defocus(self):
+        """
+        The defocus property
+
+        """
+        return self._defocus
+
     @property
     def is_mrcfile_writer(self):
         """
@@ -141,7 +149,7 @@ class MrcFileWriter(Writer):
         def __init__(self, handle):
             self.handle = handle
             n = len(self.handle.extended_header)
-            self.x, self.y = numpy.meshgrid(numpy.arange(0, 3), numpy.arange(0, n))
+            self.x, self.y = np.meshgrid(np.arange(0, 3), np.arange(0, n))
 
         def __setitem__(self, item, data):
 
@@ -151,18 +159,63 @@ class MrcFileWriter(Writer):
                     self.handle.extended_header[j]["Shift X"] = d
                 elif i == 1:
                     self.handle.extended_header[j]["Shift Y"] = d
+                elif i == 2:
+                    self.handle.extended_header[j]["Z-Stage"] = d
 
             # Get the indices from the item
             x = self.x[item]
             y = self.y[item]
 
             # Set the item
-            if isinstance(x, numpy.ndarray):
+            if isinstance(x, np.ndarray):
                 for j, i, d in zip(y, x, data):
                     setitem_internal(j, i, d)
             else:
                 setitem_internal(y, x, data)
 
+    class DriftProxy(object):
+        """
+        Proxy interface to drifts
+
+        """
+
+        def __init__(self, handle):
+            self.handle = handle
+            n = len(self.handle.extended_header)
+            self.x, self.y = np.meshgrid(np.arange(0, 2), np.arange(0, n))
+
+        def __setitem__(self, item, data):
+
+            # Set the items
+            def setitem_internal(j, i, d):
+                if i == 0:
+                    self.handle.extended_header[j]["Shift offset X"] = d
+                elif i == 1:
+                    self.handle.extended_header[j]["Shift offset Y"] = d
+
+            # Get the indices from the item
+            x = self.x[item]
+            y = self.y[item]
+
+            # Set the item
+            if isinstance(x, np.ndarray):
+                for j, i, d in zip(y, x, data):
+                    setitem_internal(j, i, d)
+            else:
+                setitem_internal(y, x, data)
+
+    class DefocusProxy(object):
+        """
+        Proxy interface to angles
+
+        """
+
+        def __init__(self, handle):
+            self.handle = handle
+
+        def __setitem__(self, item, data):
+            self.handle.extended_header[item]["Defocus"] = data
+
     def __init__(self, filename, shape, pixel_size, dtype="uint8"):
         """
         Initialise the writer
@@ -175,17 +228,17 @@ class MrcFileWriter(Writer):
 
         """
         # Get the dtype
-        dtype = numpy.dtype(dtype)
+        dtype = np.dtype(dtype)
 
         # Convert 32bit int and 64bit float
         if dtype == "int32":
-            dtype = numpy.dtype(numpy.int16)
+            dtype = np.dtype(np.int16)
         elif dtype == "uint32":
-            dtype = numpy.dtype(numpy.uint16)
+            dtype = np.dtype(np.uint16)
         elif dtype == "float64":
-            dtype = numpy.dtype(numpy.float32)
+            dtype = np.dtype(np.float32)
         elif dtype == "complex128":
-            dtype = numpy.dtype(numpy.complex64)
+            dtype = np.dtype(np.complex64)
 
         # Open the handle to the mrcfile
         self.handle = mrcfile.new_mmap(
@@ -196,7 +249,7 @@ class MrcFileWriter(Writer):
         )
 
         # Setup the extended header
-        extended_header = numpy.zeros(shape=shape[0], dtype=FEI_EXTENDED_HEADER_DTYPE)
+        extended_header = np.zeros(shape=shape[0], dtype=FEI_EXTENDED_HEADER_DTYPE)
 
         # Set the extended header
         self.handle._check_writeable()
@@ -219,6 +272,8 @@ class MrcFileWriter(Writer):
         self._data = self.handle.data
         self._angle = MrcFileWriter.AngleProxy(self.handle)
         self._position = MrcFileWriter.PositionProxy(self.handle)
+        self._drift = MrcFileWriter.DriftProxy(self.handle)
+        self._defocus = MrcFileWriter.DefocusProxy(self.handle)
 
     @property
     def pixel_size(self):
@@ -242,37 +297,6 @@ class NexusWriter(Writer):
 
     """
 
-    class ShiftProxy(object):
-        """
-        Proxy interface to positions
-
-        """
-
-        def __init__(self, handle):
-            self.handle = handle
-            n = self.handle["x_shift"].shape[0]
-            self.x, self.y = numpy.meshgrid(numpy.arange(0, 2), numpy.arange(0, n))
-
-        def __setitem__(self, item, data):
-
-            # Set the items
-            def setitem_internal(j, i, d):
-                if i == 0:
-                    self.handle["x_shift"][j] = d
-                elif i == 1:
-                    self.handle["y_shift"][j] = d
-
-            # Get the indices from the item
-            x = self.x[item]
-            y = self.y[item]
-
-            # Set the item
-            if isinstance(x, numpy.ndarray):
-                for j, i, d in zip(y, x, data):
-                    setitem_internal(j, i, d)
-            else:
-                setitem_internal(y, x, data)
-
     class PositionProxy(object):
         """
         Proxy interface to positions
@@ -282,7 +306,7 @@ class NexusWriter(Writer):
         def __init__(self, handle):
             self.handle = handle
             n = self.handle["x_translation"].shape[0]
-            self.x, self.y = numpy.meshgrid(numpy.arange(0, 3), numpy.arange(0, n))
+            self.x, self.y = np.meshgrid(np.arange(0, 3), np.arange(0, n))
 
         def __setitem__(self, item, data):
 
@@ -300,7 +324,38 @@ class NexusWriter(Writer):
             y = self.y[item]
 
             # Set the item
-            if isinstance(x, numpy.ndarray):
+            if isinstance(x, np.ndarray):
+                for j, i, d in zip(y, x, data):
+                    setitem_internal(j, i, d)
+            else:
+                setitem_internal(y, x, data)
+
+    class ShiftProxy(object):
+        """
+        Proxy interface to shifts
+
+        """
+
+        def __init__(self, handle):
+            self.handle = handle
+            n = self.handle["x_drift"].shape[0]
+            self.x, self.y = np.meshgrid(np.arange(0, 2), np.arange(0, n))
+
+        def __setitem__(self, item, data):
+
+            # Set the items
+            def setitem_internal(j, i, d):
+                if i == 0:
+                    self.handle["x_drift"][j] = d
+                elif i == 1:
+                    self.handle["y_drift"][j] = d
+
+            # Get the indices from the item
+            x = self.x[item]
+            y = self.y[item]
+
+            # Set the item
+            if isinstance(x, np.ndarray):
                 for j, i, d in zip(y, x, data):
                     setitem_internal(j, i, d)
             else:
@@ -334,20 +389,21 @@ class NexusWriter(Writer):
         detector = instrument.create_group("detector")
         detector.attrs["NX_class"] = "NXdetector"
         detector.create_dataset("data", shape=shape, dtype=dtype)
-        detector["image_key"] = numpy.zeros(shape=shape[0])
-        detector["x_pixel_size"] = numpy.full(shape=shape[0], fill_value=pixel_size)
-        detector["y_pixel_size"] = numpy.full(shape=shape[0], fill_value=pixel_size)
+        detector["image_key"] = np.zeros(shape=shape[0])
+        detector["x_pixel_size"] = np.full(shape=shape[0], fill_value=pixel_size)
+        detector["y_pixel_size"] = np.full(shape=shape[0], fill_value=pixel_size)
 
         # Create the sample
         sample = entry.create_group("sample")
         sample.attrs["NX_class"] = "NXsample"
         sample["name"] = "parakeet-simulation"
-        sample.create_dataset("rotation_angle", shape=(shape[0],), dtype=numpy.float32)
-        sample.create_dataset("x_translation", shape=(shape[0],), dtype=numpy.float32)
-        sample.create_dataset("y_translation", shape=(shape[0],), dtype=numpy.float32)
-        sample.create_dataset("z_translation", shape=(shape[0],), dtype=numpy.float32)
-        sample.create_dataset("x_shift", shape=(shape[0],), dtype=numpy.float32)
-        sample.create_dataset("y_shift", shape=(shape[0],), dtype=numpy.float32)
+        sample.create_dataset("rotation_angle", shape=(shape[0],), dtype=np.float32)
+        sample.create_dataset("x_translation", shape=(shape[0],), dtype=np.float32)
+        sample.create_dataset("y_translation", shape=(shape[0],), dtype=np.float32)
+        sample.create_dataset("z_translation", shape=(shape[0],), dtype=np.float32)
+        sample.create_dataset("x_drift", shape=(shape[0],), dtype=np.float32)
+        sample.create_dataset("y_drift", shape=(shape[0],), dtype=np.float32)
+        sample.create_dataset("defocus", shape=(shape[0],), dtype=np.float32)
 
         # Create the data
         data = entry.create_group("data")
@@ -356,15 +412,17 @@ class NexusWriter(Writer):
         data["x_translation"] = sample["x_translation"]
         data["y_translation"] = sample["y_translation"]
         data["z_translation"] = sample["z_translation"]
-        data["x_shift"] = sample["x_shift"]
-        data["y_shift"] = sample["y_shift"]
+        data["x_drift"] = sample["x_drift"]
+        data["y_drift"] = sample["y_drift"]
+        data["defocus"] = sample["defocus"]
         data["image_key"] = detector["image_key"]
 
         # Set the data ptr
         self._data = data["data"]
         self._angle = data["rotation_angle"]
         self._position = NexusWriter.PositionProxy(data)
-        self._shift = NexusWriter.ShiftProxy(data)
+        self._drift = NexusWriter.ShiftProxy(data)
+        self._defocus = data["defocus"]
 
     @property
     def pixel_size(self):
@@ -410,18 +468,18 @@ class ImageWriter(Writer):
             assert data.shape[1] == self.shape[2]
 
             # Convert to squared amplitude
-            if numpy.iscomplexobj(data):
-                data = numpy.abs(data) ** 2
+            if np.iscomplexobj(data):
+                data = np.abs(data) ** 2
                 self.vmin = None
                 self.vmax = None
 
             # Compute scale factors to put between 0 and 255
             if self.vmin is None:
-                vmin = numpy.min(data)
+                vmin = np.min(data)
             else:
                 vmin = self.vmin
             if self.vmax is None:
-                vmax = numpy.max(data)
+                vmax = np.max(data)
             else:
                 vmax = self.vmax
             s1 = 255.0 / (vmax - vmin)
@@ -429,7 +487,7 @@ class ImageWriter(Writer):
 
             # Save the image to file
             filename = self.template % (item[0] + 1)
-            image = (data * s1 + s0).astype(numpy.uint8)
+            image = (data * s1 + s0).astype(np.uint8)
             PIL.Image.fromarray(image).save(filename)
 
     def __init__(self, template, shape=None, vmin=None, vmax=None):
@@ -446,8 +504,10 @@ class ImageWriter(Writer):
         self._data = ImageWriter.DataProxy(template, shape, vmin, vmax)
 
         # Create dummy arrays for angle and position
-        self._angle = numpy.zeros(shape=shape[0], dtype=numpy.float32)
-        self._position = numpy.zeros(shape=(shape[0], 3), dtype=numpy.float32)
+        self._angle = np.zeros(shape=shape[0], dtype=np.float32)
+        self._position = np.zeros(shape=(shape[0], 3), dtype=np.float32)
+        self._drift = np.zeros(shape=(shape[0], 2), dtype=np.float32)
+        self._defocus = np.zeros(shape=shape[0], dtype=np.float32)
         self._pixel_size = 0
 
     @property
@@ -489,7 +549,9 @@ class Reader(object):
 
     """
 
-    def __init__(self, handle, data, angle, position, pixel_size):
+    def __init__(
+        self, handle, data, angle, position, pixel_size, drift=None, defocus=None
+    ):
         """
         Initialise the data
 
@@ -498,6 +560,8 @@ class Reader(object):
             angle (array): The angle array
             position (array): The position array
             pixel_size (float): The pixel size array
+            drift (float): The drift array
+            defocus (float): The defocus array
 
         """
         # Check the size
@@ -510,6 +574,8 @@ class Reader(object):
         self.angle = angle
         self.position = position
         self.pixel_size = pixel_size
+        self.drift = drift
+        self.defocus = defocus
         self.shape = data.shape
         self.dtype = data.dtype
 
@@ -543,47 +609,6 @@ class Reader(object):
             step = 1
         else:
             step = (self.angle[-1] - self.angle[0]) / (len(self.angle) - 1)
-        # assert all(
-        #     abs((b - a) - step) < tol for a, b in zip(self.angle[0:-1], self.angle[1:])
-        # )
-        return step
-
-    @property
-    def start_position(self):
-        """
-        Returns:
-            float: the start position
-
-        """
-        return self.position[0][1]
-
-    @property
-    def stop_position(self):
-        """
-        Returns:
-            float: The stop position
-
-        """
-        return self.position[-1][1]
-
-    @property
-    def step_position(self):
-        """
-        Returns:
-            float: The stop position
-
-        """
-        tol = 1e-7
-        if self.position.shape[0] > 1:
-            step = (self.position[-1, 1] - self.position[0, 1]) / (
-                self.position.shape[0] - 1
-            )
-            assert all(
-                abs((b - a) - step) < tol
-                for a, b in zip(self.position[0:-1, 1], self.position[1:1])
-            )
-        else:
-            step = 0
         return step
 
     @property
@@ -615,25 +640,45 @@ class Reader(object):
             assert handle.extended_header.shape[0] == handle.data.shape[0]
 
             # Read the angles
-            angle = numpy.zeros(handle.data.shape[0], dtype=numpy.float32)
+            angle = np.zeros(handle.data.shape[0], dtype=np.float32)
             for i in range(handle.extended_header.shape[0]):
                 angle[i] = handle.extended_header[i]["Alpha tilt"]
 
-            # Read the positions
-            position = numpy.zeros(shape=(handle.data.shape[0], 3), dtype=numpy.float32)
+            position = np.zeros(shape=(handle.data.shape[0], 3), dtype=np.float32)
+            drift = np.zeros(shape=(handle.data.shape[0], 2), dtype=np.float32)
+            defocus = np.zeros(shape=(handle.data.shape[0]), dtype=np.float32)
             for i in range(handle.extended_header.shape[0]):
+
+                # Read the positions
                 position[i, 0] = handle.extended_header[i]["Shift X"]
                 position[i, 1] = handle.extended_header[i]["Shift Y"]
-                position[i, 2] = 0
+                position[i, 2] = handle.extended_header[i]["Z-Stage"]
+
+                # Read the drift
+                drift[i, 0] = handle.extended_header[i]["Shift offset X"]
+                drift[i, 1] = handle.extended_header[i]["Shift offset Y"]
+
+                # Read the defocus
+                defocus[i] = handle.extended_header[i]["Defocus"]
         else:
-            angle = numpy.zeros(handle.data.shape[0], dtype=numpy.float32)
-            position = numpy.zeros(shape=(handle.data.shape[0], 3), dtype=numpy.float32)
+            angle = np.zeros(handle.data.shape[0], dtype=np.float32)
+            position = np.zeros(shape=(handle.data.shape[0], 3), dtype=np.float32)
+            drift = np.zeros(shape=(handle.data.shape[0], 2), dtype=np.float32)
+            defocus = np.zeros(handle.data.shape[0], dtype=np.float32)
 
         # Get the pixel size
         pixel_size = handle.voxel_size["x"]
 
         # Create the reader
-        return Reader(handle, handle.data, angle, position, pixel_size)
+        return Reader(
+            handle,
+            handle.data,
+            angle,
+            position,
+            pixel_size,
+            drift=drift,
+            defocus=defocus,
+        )
 
     @classmethod
     def from_nexus(Class, filename):
@@ -661,16 +706,28 @@ class Reader(object):
         detector = entry["instrument"]["detector"]
 
         # Get the positions
-        position = numpy.array(
+        position = np.array(
             (data["x_translation"], data["y_translation"], data["z_translation"])
         ).T
 
+        # Get the drifts
+        drift = np.array((data["x_drift"], data["y_drift"])).T
+
+        # Get the defocus
+        defocus = data.get("defocus", None)
+
         # Get the pixel size
         pixel_size = detector["x_pixel_size"][0]
 
         # Create the reader
         return Reader(
-            handle, data["data"], data["rotation_angle"], position, pixel_size
+            handle,
+            data["data"],
+            data["rotation_angle"],
+            position,
+            pixel_size,
+            drift=drift,
+            defocus=defocus,
         )
 
     @classmethod
