@@ -12,7 +12,7 @@
 import copy
 import logging
 import mrcfile
-import numpy
+import numpy as np
 import warnings
 import parakeet.config
 import parakeet.dqe
@@ -22,7 +22,6 @@ import parakeet.inelastic
 import parakeet.sample
 from math import sqrt, pi, sin, floor
 from collections.abc import Iterable
-from scipy.spatial.transform import Rotation
 
 # Try to input MULTEM
 try:
@@ -144,8 +143,8 @@ def create_input_multislice(
 
     # Microscope parameters
     input_multislice.E_0 = microscope.beam.energy
-    input_multislice.theta = 0.0
-    input_multislice.phi = 0.0
+    input_multislice.theta = microscope.beam.theta
+    input_multislice.phi = microscope.beam.phi
 
     # Illumination model
     input_multislice.illumination_model = "Partial_Coherent"
@@ -156,7 +155,7 @@ def create_input_multislice(
     ssf_sigma = multem.mrad_to_sigma(
         input_multislice.E_0, microscope.beam.source_spread
     )
-    input_multislice.cond_lens_ssf_sigma = ssf_sigma
+    input_multislice.cond_lens_si_sigma = ssf_sigma
 
     # Objective lens
     input_multislice.obj_lens_m = microscope.lens.m
@@ -193,7 +192,7 @@ def create_input_multislice(
         input_multislice.phase_shift = pi / 2.0
 
     # defocus spread function
-    input_multislice.obj_lens_dsf_sigma = multem.iehwgd_to_sigma(
+    input_multislice.obj_lens_ti_sigma = multem.iehwgd_to_sigma(
         defocus_spread(
             microscope.lens.c_c * 1e-3 / 1e-10,  # Convert from mm to A
             microscope.beam.energy_spread,
@@ -238,6 +237,10 @@ class Simulation(object):
         self.pixel_size = pixel_size
         self.image_size = image_size
         self.scan = scan
+        from .scan import UniformAngularScan
+
+        if isinstance(scan, UniformAngularScan):  # single particle mode check
+            self.scan.poses.write_star_file(self.scan.metadata_file)
         self.cluster = cluster
         self.simulate_image = simulate_image
 
@@ -279,13 +282,15 @@ class Simulation(object):
                 logger.info(
                     f"    Running job: {i+1}/{self.shape[0]} for {angle} degrees"
                 )
-                _, angle, position, image, shift = self.simulate_image(i)
-                if writer:
+                _, angle, position, image, drift, defocus = self.simulate_image(i)
+                if writer is not None:
                     writer.data[i, :, :] = image
                     writer.angle[i] = angle
-                    writer.position[i] = (0, position, 0)
-                    if shift:
-                        writer.shift[i] = shift
+                    writer.position[i] = position
+                    if drift is not None:
+                        writer.drift[i] = drift
+                    if defocus is not None:
+                        writer.defocus[i] = defocus
         else:
 
             # Set the maximum number of workers
@@ -313,17 +318,21 @@ class Simulation(object):
                 for j, future in enumerate(parakeet.futures.as_completed(futures)):
 
                     # Get the result
-                    i, angle, position, image = future.result()
+                    i, angle, position, image, drift, defocus = future.result()
 
                     # Set the output in the writer
-                    if writer:
+                    if writer is not None:
                         writer.data[i, :, :] = image
                         writer.angle[i] = angle
-                        writer.position[i] = (0, position, 0)
+                        writer.position[i] = position
+                        if drift is not None:
+                            writer.drift[i] = drift
+                        if defocus is not None:
+                            writer.defocus[i] = defocus
 
                     # Write some info
-                    vmin = numpy.min(image)
-                    vmax = numpy.max(image)
+                    vmin = np.min(image)
+                    vmax = np.max(image)
                     logger.info(
                         "    Processed job: %d (%d/%d); image min/max: %.2f/%.2f"
                         % (i + 1, j + 1, self.shape[0], vmin, vmax)
@@ -378,22 +387,9 @@ class ProjectedPotentialSimulator(object):
         # Get the specimen atoms
         logger.info(f"Simulating image {index+1}")
 
-        # Set the rotation angle
-        # input_multislice.spec_rot_theta = angle
-        # input_multislice.spec_rot_u0 = simulation.scan.axis
-
         # Create the sample extractor
         x0 = (-offset, -offset)
         x1 = (x_fov + offset, y_fov + offset)
-        # thickness = self.simulation["division_thickness"]
-        # extractor = parakeet.sample.AtomSliceExtractor(
-        #     sample=self.sample,
-        #     translation=position,
-        #     rotation=angle,
-        #     x0=x0,
-        #     x1=x1,
-        #     thickness=thickness,
-        # )
 
         # Create the multem system configuration
         system_conf = create_system_configuration(self.device)
@@ -415,17 +411,9 @@ class ProjectedPotentialSimulator(object):
         input_multislice.spec_ly = y_fov + offset * 2
         input_multislice.spec_lz = self.sample.containing_box[1][2]
 
-        # Either slice or don't
-        # assert len(extractor) == 1
-
         # Set the atoms in the input after translating them for the offset
-        # zslice = extractor[0]
         atoms = self.sample.get_atoms_in_fov(x0, x1)
         logger.info("Simulating with %d atoms" % atoms.data.shape[0])
-        # logger.info(
-        #     "    Simulating z slice %f -> %f with %d atoms"
-        #     % (zslice.x_min[2], zslice.x_max[2], zslice.atoms.data.shape[0])
-        # )
 
         # Set atom sigma
         # atoms.data["sigma"] = sigma_B
@@ -433,61 +421,48 @@ class ProjectedPotentialSimulator(object):
         if len(atoms.data) > 0:
             coords = atoms.data[["x", "y", "z"]].to_numpy()
             coords = (
-                Rotation.from_rotvec((0, angle * pi / 180, 0)).apply(
-                    coords - self.sample.centre
-                )
+                self.scan.poses.orientations[index].apply(coords - self.sample.centre)
                 + self.sample.centre
+                - self.scan.poses.shifts[index]
             ).astype("float32")
             atoms.data["x"] = coords[:, 0]
             atoms.data["y"] = coords[:, 1]
             atoms.data["z"] = coords[:, 2]
-        # atoms.data = atoms.data.append(parakeet.sample.AtomData(atomic_number=[1,1], x=[750,750], y=[750,750], z=[0,4000],sigma=[0,0],occupancy=[1,1],charge=[0,0]).data)
 
         origin = (0, 0)
         input_multislice.spec_atoms = atoms.translate(
             (offset - origin[0], offset - origin[1], 0)
         ).to_multem()
         logger.info("   Got spec atoms")
-        # logger.info(
-        #     "    Simulating z slice %f -> %f with %d atoms"
-        #     % (zslice.x_min[2], zslice.x_max[2], zslice.atoms.data.shape[0])
-        # )
-        # input_multislice.spec_atoms = zslice.atoms.translate(
-        #     (offset, offset, 0)
-        # ).to_multem()
 
         # Get the potential and thickness
         volume_z0 = self.sample.shape_box[0][2]
         volume_z1 = self.sample.shape_box[1][2]
         slice_thickness = self.simulation["slice_thickness"]
         zsize = int(floor((volume_z1 - volume_z0) / slice_thickness))
-        handle = mrcfile.new("projected_potential_%d.mrc" % index, overwrite=True)
-        potential = numpy.zeros(shape=(zsize, ny, nx), dtype="float32")
-        # thickness = handle.create_dataset(
-        #     "thickness", (0,), dtype="float32", maxshape=(None,)
-        # )
-        # potential = handle.create_dataset(
-        #     "potential", (0, 0, 0), dtype="float32", maxshape=(None, None, None)
-        # )
+        potential = mrcfile.new_mmap(
+            "projected_potential_%d.mrc" % index,
+            shape=(zsize, ny, nx),
+            mrc_mode=mrcfile.utils.mode_from_dtype(np.dtype(np.float32)),
+            overwrite=True,
+        )
+        potential.voxel_size = tuple((pixel_size, pixel_size, slice_thickness))
 
         def callback(z0, z1, V):
-            V = numpy.array(V)
+            V = np.array(V)
             zc = (z0 + z1) / 2.0
             index = int(floor((zc - volume_z0) / slice_thickness))
             print(
                 "Calculating potential for slice: %.2f -> %.2f (index: %d)"
                 % (z0, z1, index)
             )
-            potential[index, :, :] = V[margin:-margin, margin:-margin].T
+            potential.data[index, :, :] = V[margin:-margin, margin:-margin].T
 
         # Run the simulation
         multem.compute_projected_potential(system_conf, input_multislice, callback)
 
-        handle.set_data(potential)
-        handle.voxel_size = tuple((pixel_size, pixel_size, slice_thickness))
-
         # Compute the image scaled with Poisson noise
-        return (index, angle, position, None, None)
+        return (index, angle, position, None, None, None)
 
 
 class ExitWaveImageSimulator(object):
@@ -509,6 +484,112 @@ class ExitWaveImageSimulator(object):
         self.simulation = simulation
         self.device = device
 
+    def get_beam_drift(self, index, angle):
+        """
+        Get the beam drift
+
+        Returns:
+            tuple: shiftx, shifty - the beam drift
+        """
+
+        beam_drift = self.microscope.beam.drift
+        driftx = 0
+        drifty = 0
+        if beam_drift and not beam_drift["type"] is None:
+            if beam_drift["type"] == "random":
+                driftx, drifty = np.random.normal(0, beam_drift["magnitude"], size=2)
+                logger.info("Adding drift of %f, %f " % (driftx, drifty))
+            elif beam_drift["type"] == "random_smoothed":
+                if index == 0:
+
+                    def generate_smoothed_random(magnitude, num_images):
+                        drift = np.random.normal(0, magnitude, size=(num_images, 2))
+                        driftx = np.convolve(drift[:, 0], np.ones(5) / 5, mode="same")
+                        drifty = np.convolve(drift[:, 1], np.ones(5) / 5, mode="same")
+                        return driftx, drifty
+
+                    self._beam_drift = generate_smoothed_random(
+                        beam_drift["magnitude"], len(self.scan)
+                    )
+                driftx = self._beam_drift[0][index]
+                drifty = self._beam_drift[1][index]
+                logger.info("Adding drift of %f, %f " % (driftx, drifty))
+            elif beam_drift["type"] == "sinusoidal":
+                driftx = sin(angle * pi / 180) * beam_drift["magnitude"]
+                drifty = driftx
+                logger.info("Adding drift of %f, %f " % (driftx, drifty))
+            else:
+                raise RuntimeError("Unknown drift type")
+
+        # Return the beam drift
+        return driftx, drifty
+
+    def get_masker(self, index, input_multislice, pixel_size, drift, origin, offset):
+        """
+        Get the masker object for the ice specification
+
+        """
+
+        # Create the masker
+        masker = multem.Masker(input_multislice.nx, input_multislice.ny, pixel_size)
+
+        # Get the sample centre
+        shape = self.sample.shape
+        centre = np.array(self.sample.centre)
+        drift = np.array(drift)
+        shift = self.scan.poses.shifts[index]
+        detector_origin = np.array([origin[0], origin[1], 0])
+        centre = centre + offset - drift - detector_origin - shift
+
+        # Set the shape
+        if shape["type"] == "cube":
+            length = shape["cube"]["length"]
+            masker.set_cuboid(
+                (
+                    centre[0] - length / 2,
+                    centre[1] - length / 2,
+                    centre[2] - length / 2,
+                ),
+                (length, length, length),
+            )
+        elif shape["type"] == "cuboid":
+            length_x = shape["cuboid"]["length_x"]
+            length_y = shape["cuboid"]["length_y"]
+            length_z = shape["cuboid"]["length_z"]
+            masker.set_cuboid(
+                (
+                    centre[0] - length_x / 2,
+                    centre[1] - length_y / 2,
+                    centre[2] - length_z / 2,
+                ),
+                (length_x, length_y, length_z),
+            )
+        elif shape["type"] == "cylinder":
+            radius = shape["cylinder"]["radius"]
+            if not isinstance(radius, Iterable):
+                radius = [radius]
+            length = shape["cylinder"]["length"]
+            offset_x = shape["cylinder"].get("offset_x", [0] * len(radius))
+            offset_z = shape["cylinder"].get("offset_z", [0] * len(radius))
+            axis = shape["cylinder"].get("axis", (0, 1, 0))
+            masker.set_cylinder(
+                (centre[0], centre[1] - length / 2, centre[2]),
+                axis,
+                length,
+                list(radius),
+                list(offset_x),
+                list(offset_z),
+            )
+
+        # Rotate unless we have a single particle type simulation
+        if self.scan.is_uniform_angular_scan:
+            masker.set_rotation(centre, (0, 0, 0))
+        else:
+            masker.set_rotation(centre, self.scan.poses.orientations[index].as_rotvec())
+
+        # Get the masker
+        return masker
+
     def __call__(self, index):
         """
         Simulate a single frame
@@ -530,49 +611,13 @@ class ExitWaveImageSimulator(object):
         position = self.scan.positions[index]
 
         # Add the beam drift
-        if (
-            self.microscope.beam.drift
-            and not self.microscope.beam.drift["type"] is None
-        ):
-            if self.microscope.beam.drift["type"] == "random":
-                shiftx, shifty = numpy.random.normal(
-                    0, self.microscope.beam.drift["magnitude"], size=2
-                )
-                logger.info("Adding drift of %f, %f " % (shiftx, shifty))
-            elif self.microscope.beam.drift["type"] == "random_smoothed":
-                if index == 0:
-
-                    def generate_smoothed_random(magnitude, num_images):
-                        shift = numpy.random.normal(0, magnitude, size=(num_images, 2))
-                        shiftx = numpy.convolve(
-                            shift[:, 0], numpy.ones(5) / 5, mode="same"
-                        )
-                        shifty = numpy.convolve(
-                            shift[:, 1], numpy.ones(5) / 5, mode="same"
-                        )
-                        return shiftx, shifty
-
-                    self.shift = generate_smoothed_random(
-                        self.microscope.beam.drift["magnitude"], len(self.scan)
-                    )
-                shiftx = self.shift[0][index]
-                shifty = self.shift[1][index]
-                logger.info("Adding drift of %f, %f " % (shiftx, shifty))
-            elif self.microscope.beam.drift["type"] == "sinusoidal":
-                shiftx = sin(angle * pi / 180) * self.microscope.beam.drift["magnitude"]
-                shifty = shiftx
-                logger.info("Adding drift of %f, %f " % (shiftx, shifty))
-            else:
-                raise RuntimeError("Unknown drift type")
-        else:
-            shiftx = 0
-            shifty = 0
+        driftx, drifty = self.get_beam_drift(index, angle)
 
         # The field of view
         nx = self.microscope.detector.nx
         ny = self.microscope.detector.ny
         pixel_size = self.microscope.detector.pixel_size
-        origin = numpy.array(self.microscope.detector.origin)
+        origin = np.array(self.microscope.detector.origin)
         margin = self.simulation["margin"]
         padding = self.simulation["padding"]
         x_fov = nx * pixel_size
@@ -581,25 +626,6 @@ class ExitWaveImageSimulator(object):
         # padding_offset = padding * pixel_size
         offset = (padding + margin) * pixel_size
 
-        # Set the rotation angle
-        # input_multislice.spec_rot_theta = angle
-        # input_multislice.spec_rot_u0 = simulation.scan.axis
-
-        # Create the sample extractor
-        x0 = numpy.array((-margin_offset, position - margin_offset))
-        x1 = numpy.array((x_fov + margin_offset, position + y_fov + margin_offset))
-        x0 += origin
-        x1 += origin
-        # thickness = self.simulation["division_thickness"]
-        # extractor = parakeet.sample.AtomSliceExtractor(
-        #    sample=self.sample,
-        #    translation=position,
-        #    rotation=angle,
-        #    x0=x0,
-        #    x1=x1,
-        #    thickness=thickness,
-        # )
-
         # Create the multem system configuration
         system_conf = create_system_configuration(self.device)
 
@@ -624,7 +650,7 @@ class ExitWaveImageSimulator(object):
         if self.simulation["radiation_damage_model"]:
             input_multislice.static_B_factor = (
                 8
-                * pi ** 2
+                * pi**2
                 * (
                     self.simulation["sensitivity_coefficient"]
                     * self.microscope.beam.electrons_per_angstrom
@@ -634,29 +660,16 @@ class ExitWaveImageSimulator(object):
         else:
             input_multislice.static_B_factor = 0
 
-        # Either slice or don't
-        # if True:  # len(extractor) == 1:
-
         # Set the atoms in the input after translating them for the offset
-        # zslice = extractor[0]
-        atoms = self.sample.get_atoms_in_fov(x0, x1)
+        atoms = self.sample.get_atoms()
         logger.info("Simulating with %d atoms" % atoms.data.shape[0])
-        # logger.info(
-        #     "    Simulating z slice %f -> %f with %d atoms"
-        #     % (zslice.x_min[2], zslice.x_max[2], zslice.atoms.data.shape[0])
-        # )
-
-        # Set atom sigma
-        # atoms.data["sigma"] = sigma_B
-
         if len(atoms.data) > 0:
             coords = atoms.data[["x", "y", "z"]].to_numpy()
             coords = (
-                Rotation.from_rotvec((0, angle * pi / 180, 0)).apply(
-                    coords - self.sample.centre
-                )
+                self.scan.poses.orientations[index].apply(coords - self.sample.centre)
                 + self.sample.centre
-                - (shiftx, shifty + position, 0)
+                - self.scan.poses.shifts[index]
+                + np.array([driftx, drifty, 0])
             ).astype("float32")
             atoms.data["x"] = coords[:, 0]
             atoms.data["y"] = coords[:, 1]
@@ -667,64 +680,26 @@ class ExitWaveImageSimulator(object):
         ).to_multem()
         logger.info("   Got spec atoms")
 
+        print(
+            "Atoms X min/max: %.1f, %.1f"
+            % (atoms.data["x"].min(), atoms.data["x"].max())
+        )
+        print(
+            "Atoms Y min/max: %.1f, %.1f"
+            % (atoms.data["y"].min(), atoms.data["y"].max())
+        )
+        print(
+            "Atoms Z min/max: %.1f, %.1f"
+            % (atoms.data["z"].min(), atoms.data["z"].max())
+        )
+
         if self.simulation["ice"] == True:
 
-            # Create the masker
-            masker = multem.Masker(input_multislice.nx, input_multislice.ny, pixel_size)
-
-            # Get the sample centre
-            shape = self.sample.shape
-            centre = self.sample.centre
-            centre = (
-                centre[0] + offset - shiftx - origin[0],
-                centre[1] + offset - shifty - position - origin[1],
-                centre[2],
+            # Get the masker
+            masker = self.get_masker(
+                index, input_multislice, pixel_size, (driftx, drifty, 0), origin, offset
             )
 
-            # Set the shape
-            if shape["type"] == "cube":
-                length = shape["cube"]["length"]
-                masker.set_cuboid(
-                    (
-                        centre[0] - length / 2,
-                        centre[1] - length / 2,
-                        centre[2] - length / 2,
-                    ),
-                    (length, length, length),
-                )
-            elif shape["type"] == "cuboid":
-                length_x = shape["cuboid"]["length_x"]
-                length_y = shape["cuboid"]["length_y"]
-                length_z = shape["cuboid"]["length_z"]
-                masker.set_cuboid(
-                    (
-                        centre[0] - length_x / 2,
-                        centre[1] - length_y / 2,
-                        centre[2] - length_z / 2,
-                    ),
-                    (length_x, length_y, length_z),
-                )
-            elif shape["type"] == "cylinder":
-                radius = shape["cylinder"]["radius"]
-                if not isinstance(radius, Iterable):
-                    radius = [radius]
-                length = shape["cylinder"]["length"]
-                offset_x = shape["cylinder"].get("offset_x", [0] * len(radius))
-                offset_z = shape["cylinder"].get("offset_z", [0] * len(radius))
-                axis = shape["cylinder"].get("axis", (0, 1, 0))
-                masker.set_cylinder(
-                    (centre[0], centre[1] - length / 2, centre[2]),
-                    axis,
-                    length,
-                    list(radius),
-                    list(offset_x),
-                    list(offset_z),
-                )
-
-            # Rotate
-            origin = centre
-            masker.set_rotation(origin, (0, angle * pi / 180.0, 0))
-
             # Run the simulation
             output_multislice = multem.simulate(system_conf, input_multislice, masker)
 
@@ -734,76 +709,19 @@ class ExitWaveImageSimulator(object):
             logger.info("Simulating")
             output_multislice = multem.simulate(system_conf, input_multislice)
 
-        # else:
-        #     pass
-        # # Slice the specimen atoms
-        # def slice_generator(extractor):
-
-        #     # Get the data from the data buffer and return
-        #     def prepare(data_buffer):
-
-        #         # Extract the data
-        #         atoms = parakeet.sample.AtomData(
-        #             data=pandas.concat([d.atoms.data for d in data_buffer])
-        #         )
-        #         z_min = min([d.x_min[2] for d in data_buffer])
-        #         z_max = max([d.x_max[2] for d in data_buffer])
-        #         assert z_min < z_max
-
-        #         # Print some info
-        #         logger.info(
-        #             "    Simulating z slice %f -> %f with %d atoms"
-        #             % (z_min, z_max, atoms.data.shape[0])
-        #         )
-
-        #         # Cast the atoms
-        #         atoms = atoms.translate((offset, offset, 0)).to_multem()
-
-        #         # Return the Z-min, Z-max and atoms
-        #         return (z_min, z_max, atoms)
-
-        #     # Loop through the slices and gather atoms until we have more
-        #     # than the maximum buffer size. There seems to be an overhead
-        #     # to the simulation code so it's better to have as many atoms
-        #     # as possible before calling. Doing this is much fast than
-        #     # simulating with only a small number of atoms.
-        #     max_buffer = 10_000_000
-        #     data_buffer = []
-        #     for zslice in extractor:
-        #         data_buffer.append(zslice)
-        #         if sum(d.atoms.data.shape[0] for d in data_buffer) > max_buffer:
-        #             yield prepare(data_buffer)
-        #             data_buffer = []
-
-        #     # Simulate from the final buffer
-        #     if len(data_buffer) > 0:
-        #         yield prepare(data_buffer)
-        #         data_buffer = []
-
-        # Run the simulation
-        # st = time.time()
-        # output_multislice = multem.simulate(
-        #     system_conf, input_multislice, slice_generator(extractor)
-        # )
-        # logger.info(
-        #     "    Image %d simulated in %d seconds" % (index, time.time() - st)
-        # )
-
         # Get the ideal image data
         # Multem outputs data in column major format. In C++ and Python we
         # generally deal with data in row major format so we must do a
         # transpose here.
-        image = numpy.array(output_multislice.data[0].psi_coh).T
+        image = np.array(output_multislice.data[0].psi_coh).T
         image = image[padding:-padding, padding:-padding]
 
         # Print some info
-        psi_tot = numpy.abs(image) ** 2
-        logger.info(
-            "Ideal image min/max: %f/%f" % (numpy.min(psi_tot), numpy.max(psi_tot))
-        )
+        psi_tot = np.abs(image) ** 2
+        logger.info("Ideal image min/max: %f/%f" % (np.min(psi_tot), np.max(psi_tot)))
 
         # Compute the image scaled with Poisson noise
-        return (index, angle, position, image, (shiftx, shifty))
+        return (index, angle, position, image, (driftx, drifty), None)
 
 
 class OpticsImageSimulator(object):
@@ -845,11 +763,52 @@ class OpticsImageSimulator(object):
 
         """
 
-        def compute_image(psi, microscope, simulation, x_fov, y_fov, offset, device):
+        def get_defocus(index, angle):
+            """
+            Get the defocus
+
+            Returns:
+                float: defocus
+            """
+
+            defocus = self.microscope.lens.c_10
+            drift = 0
+            defocus_drift = self.microscope.beam.defocus_drift
+            if defocus_drift and not defocus_drift["type"] is None:
+                if defocus_drift["type"] == "random":
+                    drift = np.random.normal(0, defocus_drift["magnitude"])
+                elif defocus_drift["type"] == "random_smoothed":
+                    if index == 0:
+
+                        def generate_smoothed_random(magnitude, num_images):
+                            drift = np.random.normal(0, magnitude, size=(num_images))
+                            drift = np.convolve(drift, np.ones(5) / 5, mode="same")
+                            return drift
+
+                        self._defocus_drift = generate_smoothed_random(
+                            defocus_drift["magnitude"], len(self.scan)
+                        )
+                    drift = self._defocus_drift[index]
+                elif defocus_drift["type"] == "sinusoidal":
+                    drift = sin(angle * pi / 180) * defocus_drift["magnitude"]
+                else:
+                    raise RuntimeError("Unknown drift type")
+                logger.info("Adding defocus drift of %f" % (drift))
+
+            # Return the defocus
+            return defocus + drift
+
+        def compute_image(
+            psi, microscope, simulation, x_fov, y_fov, offset, device, defocus=None
+        ):
 
             # Create the multem system configuration
             system_conf = create_system_configuration(device)
 
+            # Set the defocus
+            if defocus is not None:
+                microscope.lens.c_10 = defocus
+
             # Create the multem input multislice object
             input_multislice = create_input_multislice(
                 microscope, simulation["slice_thickness"], simulation["margin"], "HRTEM"
@@ -861,7 +820,7 @@ class OpticsImageSimulator(object):
             input_multislice.spec_lz = x_fov  # self.sample.containing_box[1][2]
 
             # Compute and apply the CTF
-            ctf = numpy.array(multem.compute_ctf(system_conf, input_multislice)).T
+            ctf = np.array(multem.compute_ctf(system_conf, input_multislice)).T
 
             # Compute the B factor for radiation damage
             # if simulation["radiation_damage_model"]:
@@ -871,27 +830,27 @@ class OpticsImageSimulator(object):
             #        * (index + 1)
             #    )
             #    pixel_size = microscope.detector.pixel_size
-            #    Y, X = numpy.mgrid[0 : ctf.shape[0], 0 : ctf.shape[1]]
+            #    Y, X = np.mgrid[0 : ctf.shape[0], 0 : ctf.shape[1]]
             #    X = (X - ctf.shape[1] // 2) / (pixel_size * ctf.shape[1])
             #    Y = (Y - ctf.shape[0] // 2) / (pixel_size * ctf.shape[0])
-            #    q = numpy.sqrt(X ** 2 + Y ** 2)
-            #    b_factor_blur = numpy.exp(-2 * pi ** 2 * q ** 2 * sigma_B ** 2)
-            #    b_factor_blur = numpy.fft.fftshift(b_factor_blur)
+            #    q = np.sqrt(X ** 2 + Y ** 2)
+            #    b_factor_blur = np.exp(-2 * pi ** 2 * q ** 2 * sigma_B ** 2)
+            #    b_factor_blur = np.fft.fftshift(b_factor_blur)
             #    ctf = ctf * b_factor_blur
 
             # Compute and apply the CTF
-            psi = numpy.fft.ifft2(numpy.fft.fft2(psi) * ctf)
-            image = numpy.abs(psi) ** 2
+            psi = np.fft.ifft2(np.fft.fft2(psi) * ctf)
+            image = np.abs(psi) ** 2
 
             return image
 
         # Get the rotation angle
-        angle = self.exit_wave.angle[index]
+        angle = self.scan.angles[index]
         position = self.scan.positions[index]
 
         # Check the angle and position
-        # assert abs(angle - self.exit_wave.angle[index]) < 1e7
-        assert (numpy.abs(position - self.exit_wave.position[index]) < 1e7).all()
+        assert abs(angle - self.exit_wave.angle[index]) < 1e7
+        assert (np.abs(position - self.exit_wave.position[index]) < 1e7).all()
 
         # The field of view
         nx = self.microscope.detector.nx
@@ -908,8 +867,14 @@ class OpticsImageSimulator(object):
         # Set the input wave
         psi = self.exit_wave.data[index]
 
+        # Get the beam drift
+        driftx, drifty = self.exit_wave.drift[index, :]
+
         microscope = copy.deepcopy(self.microscope)
 
+        # Get the defocus
+        defocus = get_defocus(index, angle)
+
         # If we do CC correction then set spherical aberration and chromatic
         # aberration to zero
         shape = self.sample["shape"]
@@ -917,7 +882,14 @@ class OpticsImageSimulator(object):
 
             # If no inelastic model just calculate image as normal
             image = compute_image(
-                psi, microscope, self.simulation, x_fov, y_fov, offset, self.device
+                psi,
+                microscope,
+                self.simulation,
+                x_fov,
+                y_fov,
+                offset,
+                self.device,
+                defocus,
             )
             electron_fraction = 1.0
 
@@ -925,7 +897,14 @@ class OpticsImageSimulator(object):
 
             # Compute the image
             image = compute_image(
-                psi, microscope, self.simulation, x_fov, y_fov, offset, self.device
+                psi,
+                microscope,
+                self.simulation,
+                x_fov,
+                y_fov,
+                offset,
+                self.device,
+                defocus,
             )
 
             # Calculate the fraction of electrons in the zero loss peak
@@ -981,7 +960,14 @@ class OpticsImageSimulator(object):
 
             # Compute the zero loss image
             image1 = compute_image(
-                psi, microscope, self.simulation, x_fov, y_fov, offset, self.device
+                psi,
+                microscope,
+                self.simulation,
+                x_fov,
+                y_fov,
+                offset,
+                self.device,
+                defocus,
             )
 
             # Add the energy loss
@@ -996,7 +982,14 @@ class OpticsImageSimulator(object):
 
             # Compute the MPL image
             image2 = compute_image(
-                psi, microscope, self.simulation, x_fov, y_fov, offset, self.device
+                psi,
+                microscope,
+                self.simulation,
+                x_fov,
+                y_fov,
+                offset,
+                self.device,
+                defocus,
             )
 
             # Compute the zero loss and mpl image fraction
@@ -1016,7 +1009,14 @@ class OpticsImageSimulator(object):
 
             # Compute the zero loss image
             image1 = compute_image(
-                psi, microscope, self.simulation, x_fov, y_fov, offset, self.device
+                psi,
+                microscope,
+                self.simulation,
+                x_fov,
+                y_fov,
+                offset,
+                self.device,
+                defocus,
             )
 
             # Add the energy loss
@@ -1029,7 +1029,14 @@ class OpticsImageSimulator(object):
 
             # Compute the MPL image
             image2 = compute_image(
-                psi, microscope, self.simulation, x_fov, y_fov, offset, self.device
+                psi,
+                microscope,
+                self.simulation,
+                x_fov,
+                y_fov,
+                offset,
+                self.device,
+                defocus,
             )
 
             # Compute the zero loss and mpl image fraction
@@ -1056,7 +1063,14 @@ class OpticsImageSimulator(object):
 
             # Compute the zero loss image
             image1 = compute_image(
-                psi, microscope, self.simulation, x_fov, y_fov, offset, self.device
+                psi,
+                microscope,
+                self.simulation,
+                x_fov,
+                y_fov,
+                offset,
+                self.device,
+                defocus,
             )
 
             # Add the energy loss
@@ -1069,7 +1083,14 @@ class OpticsImageSimulator(object):
 
             # Compute the MPL image
             image2 = compute_image(
-                psi, microscope, self.simulation, x_fov, y_fov, offset, self.device
+                psi,
+                microscope,
+                self.simulation,
+                x_fov,
+                y_fov,
+                offset,
+                self.device,
+                defocus,
             )
 
             # Compute the zero loss and mpl image fraction
@@ -1097,11 +1118,11 @@ class OpticsImageSimulator(object):
         image = image[j0:j1, i0:i1]
         logger.info(
             "Ideal image min/mean/max: %f/%f/%f"
-            % (numpy.min(image), numpy.mean(image), numpy.max(image))
+            % (np.min(image), np.mean(image), np.max(image))
         )
 
         # Compute the image scaled with Poisson noise
-        return (index, angle, position, image, None)
+        return (index, angle, position, image, (driftx, drifty), defocus)
 
 
 class ImageSimulator(object):
@@ -1137,19 +1158,12 @@ class ImageSimulator(object):
         """
 
         # Get the rotation angle
-        angle = self.optics.angle[index]
+        angle = self.scan.angles[index]
         position = self.scan.positions[index]
 
         # Check the angle and position
-        # assert abs(angle - self.optics.angle[index]) < 1e7
-        assert (numpy.abs(position - self.optics.position[index]) < 1e7).all()
-
-        # The field of view
-        nx = self.microscope.detector.nx
-        ny = self.microscope.detector.ny
-        pixel_size = self.microscope.detector.pixel_size
-        # x_fov = nx * pixel_size
-        # y_fov = ny * pixel_size
+        assert abs(angle - self.optics.angle[index]) < 1e7
+        assert (np.abs(position - self.optics.position[index]) < 1e7).all()
 
         # Get the specimen atoms
         logger.info(f"Simulating image {index+1}")
@@ -1157,7 +1171,7 @@ class ImageSimulator(object):
         # Compute the number of counts per pixel
         electrons_per_pixel = (
             self.microscope.beam.electrons_per_angstrom
-            * self.microscope.detector.pixel_size ** 2
+            * self.microscope.detector.pixel_size**2
         )
 
         # Compute the electrons per pixel second
@@ -1167,37 +1181,36 @@ class ImageSimulator(object):
         # Get the image
         image = self.optics.data[index]
 
+        # Get some other properties to propagate
+        beam_drift = self.optics.drift[index]
+        defocus = self.optics.defocus[index]
+
         # Apply the dqe in Fourier space
         if self.microscope.detector.dqe:
             logger.info("Applying DQE")
             dqe = parakeet.dqe.DQETable().dqe_fs(
                 energy, electrons_per_second, image.shape
             )
-            dqe = numpy.fft.fftshift(dqe)
-            fft_image = numpy.fft.fft2(image)
+            dqe = np.fft.fftshift(dqe)
+            fft_image = np.fft.fft2(image)
             fft_image *= dqe
-            image = numpy.real(numpy.fft.ifft2(fft_image))
+            image = np.real(np.fft.ifft2(fft_image))
 
-        # Normalize so that the average pixel value is 1.0
-        image = numpy.clip(image, 0, None)
-        # image = image / numpy.mean(image) # Dont do this
+        # Ensure all pixels are >= 0
+        image = np.clip(image, 0, None)
 
         # Add Poisson noise
-        # numpy.random.seed(index)
-        image = numpy.random.poisson(image * electrons_per_pixel).astype("float64")
+        # np.random.seed(index)
+        image = np.random.poisson(image * electrons_per_pixel).astype("float64")
 
         # Print some info
         logger.info(
             "    Image min/max/mean: %g/%g/%.2g"
-            % (numpy.min(image), numpy.max(image), numpy.mean(image))
+            % (np.min(image), np.max(image), np.mean(image))
         )
 
-        # Normalize image again
-        # if electrons_per_pixel > 0:
-        #     image = image / electrons_per_pixel
-
         # Compute the image scaled with Poisson noise
-        return (index, angle, position, image.astype("float32"), None)
+        return (index, angle, position, image.astype("float32"), beam_drift, defocus)
 
 
 class CTFSimulator(object):
@@ -1253,11 +1266,11 @@ class CTFSimulator(object):
         input_multislice.spec_lz = x_fov  # self.sample.containing_box[1][2]
 
         # Run the simulation
-        image = numpy.array(multem.compute_ctf(system_conf, input_multislice)).T
-        image = numpy.fft.fftshift(image)
+        image = np.array(multem.compute_ctf(system_conf, input_multislice)).T
+        image = np.fft.fftshift(image)
 
         # Compute the image scaled with Poisson noise
-        return (index, 0, 0, image, None)
+        return (index, 0, 0, image, None, None)
 
 
 class SimpleImageSimulator(object):
@@ -1325,7 +1338,7 @@ class SimpleImageSimulator(object):
         # Set the specimen size
         input_multislice.spec_lx = x_fov + offset * 2
         input_multislice.spec_ly = y_fov + offset * 2
-        input_multislice.spec_lz = numpy.max(self.atoms.data["z"])
+        input_multislice.spec_lz = np.max(self.atoms.data["z"])
 
         # Set the atoms in the input after translating them for the offset
         input_multislice.spec_atoms = self.atoms.translate(
@@ -1339,16 +1352,14 @@ class SimpleImageSimulator(object):
         # Multem outputs data in column major format. In C++ and Python we
         # generally deal with data in row major format so we must do a
         # transpose here.
-        image = numpy.array(output_multislice.data[0].psi_coh).T
+        image = np.array(output_multislice.data[0].psi_coh).T
 
         # Print some info
-        psi_tot = numpy.abs(image) ** 2
-        logger.info(
-            "Ideal image min/max: %f/%f" % (numpy.min(psi_tot), numpy.max(psi_tot))
-        )
+        psi_tot = np.abs(image) ** 2
+        logger.info("Ideal image min/max: %f/%f" % (np.min(psi_tot), np.max(psi_tot)))
 
         # Compute the image scaled with Poisson noise
-        return (index, angle, position, image, None)
+        return (index, angle, position, image, None, None)
 
 
 def projected_potential(
